@@ -2,7 +2,7 @@ import sys, re
 
 import pandas as pd
 
-from .utils import decode_harmonies, is_any_row_equal, html2format, load_tsv, map_dict, name2format, resolve_dir, rgb2format, update_cfg
+from .utils import DCML_REGEX, DCML_DOUBLE_REGEX, decode_harmonies, is_any_row_equal, html2format, load_tsv, map_dict, name2format, resolve_dir, rgb2format, update_cfg
 from .logger import LoggedClass
 from .expand_dcml import expand_labels
 
@@ -10,65 +10,9 @@ class Annotations(LoggedClass):
     """
     Class for storing, converting and manipulating annotation labels.
     """
-    dcml_double_re = re.compile(r"""
-                                    ^(?P<first>
-                                      (\.?
-                                        ((?P<globalkey>[a-gA-G](b*|\#*))\.)?
-                                        ((?P<localkey>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
-                                        ((?P<pedal>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
-                                        (?P<chord>
-                                            (?P<numeral>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
-                                            (?P<form>(%|o|\+|M|\+M))?
-                                            (?P<figbass>(7|65|43|42|2|64|6))?
-                                            (\((?P<changes>((\+|-|\^)?(b*|\#*)\d)+)\))?
-                                            (/(?P<relativeroot>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
-                                        )
-                                        (?P<pedalend>\])?
-                                      )?
-                                      (?P<phraseend>(\\\\|\{|\}|\}\{)
-                                      )?
-                                     )
-                                     (?P<second>
-                                      (-
-                                        ((?P<globalkey2>[a-gA-G](b*|\#*))\.)?
-                                        ((?P<localkey2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
-                                        ((?P<pedal2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
-                                        (?P<chord2>
-                                            (?P<numeral2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
-                                            (?P<form2>(%|o|\+|M|\+M))?
-                                            (?P<figbass2>(7|65|43|42|2|64|6))?
-                                            (\((?P<changes2>((\+|-|\^)?(b*|\#*)\d)+)\))?
-                                            (/(?P<relativeroot2>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
-                                        )
-                                        (?P<pedalend2>\])?
-                                      )?
-                                      (?P<phraseend2>(\\\\|\{|\}|\}\{)
-                                      )?
-                                     )?
-                                    $
-                                    """,
-                            re.VERBOSE)
-
-    dcml_re = re.compile(r"""^(\.?
-                                ((?P<globalkey>[a-gA-G](b*|\#*))\.)?
-                                ((?P<localkey>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
-                                ((?P<pedal>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
-                                (?P<chord>
-                                    (?P<numeral>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
-                                    (?P<form>(%|o|\+|M|\+M))?
-                                    (?P<figbass>(7|65|43|42|2|64|6))?
-                                    (\((?P<changes>((\+|-|\^)?(b*|\#*)\d)+)\))?
-                                    (/(?P<relativeroot>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
-                                )
-                                (?P<pedalend>\])?
-                              )?
-                              (?P<phraseend>(\\\\|\{|\}|\}\{)
-                              )?$
-                            """,
-                    re.VERBOSE)
 
     main_cols = ['label', 'mc', 'mc_onset', 'staff', 'voice']
-    additional_cols = ['label_type', 'root', 'rootCase', 'base', 'leftParen', 'rightParen', 'offset_x', 'offset_y',
+    additional_cols = ['label_type', 'absolute_root', 'rootCase', 'absolute_base', 'leftParen', 'rightParen', 'offset_x', 'offset_y',
                        'nashville', 'decoded', 'color_name', 'color_html', 'color_r', 'color_g', 'color_b', 'color_a']
 
     def __init__(self, tsv_path=None, df=None, cols={}, index_col=None, sep='\t', mscx_obj=None, infer_types={}, read_only=False, logger_cfg={}, **kwargs):
@@ -151,7 +95,8 @@ Possible values are {{1, 2, 3, 4}}.""")
             return pd.DataFrame()
 
         if self.cols['mc'] not in cols:
-            if self.cols['mn'] not in cols:
+            mn_col = self.cols['mn'] if 'mn' in self.cols else 'mn'
+            if mn_col not in cols:
                 self.logger.warning(f"Annotations are lacking 'mn' and 'mc' columns.")
                 error = True
             else:
@@ -331,10 +276,10 @@ Possible values are {{1, 2, 3, 4}}.""")
             Expanded DCML labels
         """
         if 'dcml' not in self.regex_dict:
-            self.regex_dict = dict(dcml=self.dcml_double_re, **self.regex_dict)
+            self.regex_dict = dict(dcml=DCML_DOUBLE_REGEX, **self.regex_dict)
             self.infer_types()
         df = self.get_labels(**kwargs)
-        sel = df.label_type.str.contains('dcml')
+        sel = df.label_type.astype(str).str.contains('dcml').fillna(False)
         if not sel.any():
             self.logger.info(f"Score does not contain any DCML harmonic annotations.")
             return
@@ -344,7 +289,7 @@ Possible values are {{1, 2, 3, 4}}.""")
             self.logger.warning(f"Score contains {(~sel).sum()} labels that don't (and {sel.sum()} that do) match the DCML standard:\n{decode_harmonies(df[~sel], keep_type=True)[['mc', 'mn', 'label', 'label_type']].to_string()}")
         df = df[sel]
         try:
-            exp = expand_labels(df, column='label', regex=self.dcml_re, chord_tones=True, logger=self.logger)
+            exp = expand_labels(df, column='label', regex=DCML_REGEX, chord_tones=True, logger=self.logger)
             if drop_others:
                 self._expanded = exp
             else:
@@ -366,7 +311,8 @@ Possible values are {{1, 2, 3, 4}}.""")
             return False
 
         mscx = mscx_obj if mscx_obj is not None else self.mscx_obj
-        cols = [self.cols[c] for c in ['mn', 'mc_onset', 'volta'] if c in self.df.columns]
+        column_names = [self.cols[c] if c in self.cols else c for c in ['mn', 'mn_onset', 'volta']]
+        cols = [c for c in column_names if c in self.df.columns]
         inferred_positions = [mscx.infer_mc(**dict(zip(cols, t))) for t in self.df[cols].values]
         return pd.DataFrame(inferred_positions, index=self.df.index, columns=['mc', 'mc_onset'])
 
