@@ -1,8 +1,11 @@
 import os, re
+from contextlib import contextmanager
+from tempfile import NamedTemporaryFile as Temp
 
 import pandas as pd
 
-from .utils import check_labels, color2rgba, decode_harmonies, no_collections_no_booleans, resolve_dir, rgba, rgba2attrs, rgba2params, unpack_mscz, update_labels_cfg, update_cfg
+from .utils import check_labels, color2rgba, convert, DCML_DOUBLE_REGEX, decode_harmonies, get_ms_version, get_musescore, no_collections_no_booleans,\
+    resolve_dir, rgba, rgba2attrs, rgba2params, test_binary, unpack_mscz, update_labels_cfg, update_cfg
 from .bs4_parser import _MSCX_bs4
 from .annotations import Annotations
 from .logger import LoggedClass
@@ -18,49 +21,6 @@ class Score(LoggedClass):
     recognizes absolute chord symbols in their decoded (string) form; they start with a note name.
     """
 
-    DCML_REGEX = re.compile(r"""
-                                ^(?P<first>
-                                  (\.?
-                                    ((?P<globalkey>[a-gA-G](b*|\#*))\.)?
-                                    ((?P<localkey>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
-                                    ((?P<pedal>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
-                                    (?P<chord>
-                                        (?P<numeral>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
-                                        (?P<form>(%|o|\+|M|\+M))?
-                                        (?P<figbass>(7|65|43|42|2|64|6))?
-                                        (\((?P<changes>((\+|-|\^|v)?(b*|\#*)\d)+)\))?
-                                        (/(?P<relativeroot>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
-                                    )
-                                    (?P<pedalend>\])?
-                                  )?
-                                  (?P<phraseend>(\\\\|\{|\}|\}\{)
-                                  )?
-                                 )
-                                 (-
-                                  (?P<second>
-                                    ((?P<globalkey2>[a-gA-G](b*|\#*))\.)?
-                                    ((?P<localkey2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
-                                    ((?P<pedal2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
-                                    (?P<chord2>
-                                        (?P<numeral2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
-                                        (?P<form2>(%|o|\+|M|\+M))?
-                                        (?P<figbass2>(7|65|43|42|2|64|6))?
-                                        (\((?P<changes2>((\+|-|\^|v)?(b*|\#*)\d)+)\))?
-                                        (/(?P<relativeroot2>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
-                                    )
-                                    (?P<pedalend2>\])?
-                                  )?
-                                  (?P<phraseend2>(\\\\|\{|\}|\}\{)
-                                  )?
-                                 )?
-                                $
-                                """,
-                            re.VERBOSE)
-    """:obj:`str`
-    Class variable with a regular expression that
-    recognizes labels conforming to the DCML harmony annotation standard.
-    """
-
 
     NASHVILLE_REGEX = r"^(b*|#*)(\d).*$"
     """:obj:`str`
@@ -75,8 +35,18 @@ class Score(LoggedClass):
     als DCML harmony annotations.
     """
 
+    native_formats = ('mscx', 'mscz')
+    """:obj:`tuple`
+    Formats that MS3 reads without having to convert.
+    """
+
+    convertible_formats = ('cap', 'capx', 'midi', 'mid', 'musicxml', 'mxl', 'xml', )
+    """:obj:`tuple`
+    Formats that have to be converted before parsing.
+    """
+
     def __init__(self, musescore_file=None, infer_label_types=['dcml'], read_only=False, labels_cfg={}, logger_cfg={},
-                 parser='bs4'):
+                 parser='bs4', ms=None):
         """
 
         Parameters
@@ -100,6 +70,10 @@ class Score(LoggedClass):
             'file': PATH_TO_LOGFILE to store all log messages under the given path.
         parser : 'bs4', optional
             The only XML parser currently implemented is BeautifulSoup 4.
+        ms : :obj:`str`, optional
+            If you want to parse musicXML files or MuseScore 2 files by temporarily converting them, pass the path or command
+            of your local MuseScore 3 installation. If you're using the standard path, you may try 'auto', or 'win' for
+            Windows, 'mac' for MacOS, or 'mscore' for Linux.
         """
         super().__init__(subclass='Score', logger_cfg=logger_cfg)
 
@@ -133,6 +107,10 @@ class Score(LoggedClass):
         Handled internally by :py:meth:`~ms3.score.Score._handle_path`.
         """
 
+        self.ms = get_musescore(ms, logger=self.logger)
+        """:obj:`str`
+        Path or command of the local MuseScore 3 installation if specified by the user."""
+
         self._mscx = None
         """:obj:`MSCX`
         The object representing the parsed MuseScore file.
@@ -166,14 +144,14 @@ class Score(LoggedClass):
             1: self.RN_REGEX,
             2: self.NASHVILLE_REGEX,
             3: self.ABS_REGEX,
-            'dcml': self.DCML_REGEX,
+            'dcml': DCML_DOUBLE_REGEX,
         }
         """:obj:`dict`
         Mapping label types to their corresponding regex. Managed via the property :py:meth:`infer_label_types`.
-        1: self.rn_regex,
-        2: self.nashville_regex,
-        3: self.abs_regex,
-        'dcml': self.dcml_regex,
+        1: self.RN_REGEX,
+        2: self.NASHVILLE_REGEX,
+        3: self.ABS_REGEX,
+        'dcml': utils.DCML_REGEX,
         """
 
         self.labels_cfg = {
@@ -286,21 +264,22 @@ class Score(LoggedClass):
         :obj:`int`
             Number of labels that were to be attached.
         """
+        assert self._mscx is not None, "No score has been parsed yet."
         assert key != 'annotations', "Labels with key 'annotations' are already attached."
         if key not in self._detached_annotations:
-            self.logger.info(f"""Key '{key}' doesn't correspond to a detached set of annotations.
+            self.mscx.logger.info(f"""Key '{key}' doesn't correspond to a detached set of annotations.
 Use one of the existing keys or load a new set with the method load_annotations().\nExisting keys: {list(self._detached_annotations.keys())}""")
             return 0, 0
 
         annotations = self._detached_annotations[key]
         goal = len(annotations.df)
         if goal == 0:
-            self.logger.warning(f"The Annotation object '{key}' does not contain any labels.")
+            self.mscx.logger.warning(f"The Annotation object '{key}' does not contain any labels.")
             return 0, 0
         df = annotations.prepare_for_attaching(staff=staff, voice=voice, check_for_clashes=check_for_clashes)
         reached = len(df)
         if reached == 0:
-            self.logger.error(f"No labels from '{key}' have been attached due to aforementioned errors.")
+            self.mscx.logger.error(f"No labels from '{key}' have been attached due to aforementioned errors.")
             return reached, goal
 
         prepared_annotations = Annotations(df=df, cols=annotations.cols, infer_types=annotations.regex_dict)
@@ -308,9 +287,9 @@ Use one of the existing keys or load a new set with the method load_annotations(
         if remove_detached:
             if reached == goal:
                 del(self._detached_annotations[key])
-                self.logger.debug(f"Detached annotations '{key}' successfully attached and removed.")
+                self.mscx.logger.debug(f"Detached annotations '{key}' successfully attached and removed.")
             else:
-                self.logger.info(f"Only {reached} of the {goal} targeted labels could be attached, so '{key}' was not removed.")
+                self.mscx.logger.info(f"Only {reached} of the {goal} targeted labels could be attached, so '{key}' was not removed.")
         return reached, goal
 
 
@@ -337,7 +316,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
             Labels not matching the regex.
         """
         if keys == 'annotations' and not self.mscx.has_annotations:
-            self.logger.debug("Score contains no Annotations.")
+            self.mscx.logger.debug("Score contains no Annotations.")
             return
         if regex is None:
             if label_type in self._label_regex:
@@ -356,7 +335,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
         for k in keys:
             (existing if k in self else missing).append(k)
         if len(missing) > 0:
-            self.logger.warning(f"The following keys are not among the Annotations objects, which are: {list(self)}")
+            self.logger.warning(f"The keys {missing} are not among the Annotations objects, which are: {list(self)}")
         if len(existing) == 0:
             return pd.DataFrame()
         labels_cfg = self.labels_cfg.copy()
@@ -407,6 +386,9 @@ Use one of the existing keys or load a new set with the method load_annotations(
         unchanged = old_vals.intersection(new_vals)
         changes_old = old_vals - unchanged
         changes_new = new_vals - unchanged
+        if len(changes_new) == 0 and len(changes_old) == 0:
+            self.mscx.logger.info(f"Comparison yielded no changes.")
+            return False
 
         new_rgba =  color2rgba(new_color)
         new_color_params = rgba2params(new_rgba)
@@ -428,12 +410,15 @@ Use one of the existing keys or load a new set with the method load_annotations(
         df = pd.DataFrame(changes_old, columns=compare_cols)
         for k, v in added_color_params.items():
             df[k] = v
-        added_changes = self.mscx.add_labels(Annotations(df=df))
-        if added_changes > 0 or added_changes > 0:
+        added_changes = self.mscx.add_labels(Annotations(df=df), )
+        if (added_changes is None or added_changes > 0) or color_changes > 0:
             self.mscx.changed = True
             self.mscx.parsed.parse_measures()
             self.mscx._update_annotations()
-            self.logger.debug(f"{color_changes} attached labels changed to {change_to}, {added_changes} labels added in {added_color}.")
+            self.mscx.logger.info(f"{color_changes} attached labels changed to {change_to}, {added_changes} labels added in {added_color}.")
+            return True
+        return False
+
 
 
     def detach_labels(self, key, staff=None, voice=None, label_type=None, delete=True):
@@ -463,18 +448,20 @@ Use one of the existing keys or load a new set with the method load_annotations(
             of the annotations for storing them separately but without removing the labels from the score.
         """
         if not self.mscx.has_annotations:
-            self.logger.info("No annotations present in score.")
+            self.mscx.logger.info("No annotations present in score.")
             return
         assert key != 'annotations', "The key 'annotations' is reserved, please choose a different one."
         if not key.isidentifier():
             self.logger.warning(
-                f"'{key}' can not be used as an identifier. The extracted labels need to be accessed via self._detached_annotations['{key}']")
+                f"'{key}' cannot be used as an identifier. The extracted labels need to be accessed via self._detached_annotations['{key}']")
         df = self.annotations.get_labels(staff=staff, voice=voice, label_type=label_type, drop=delete)
         if len(df) == 0:
-            self.logger.info(f"No labels found for staff {staff}, voice {voice}, label_type {label_type}.")
+            self.mscx.logger.info(f"No labels found for staff {staff}, voice {voice}, label_type {label_type}.")
             return
         logger_cfg = self.logger_cfg.copy()
-        logger_cfg['name'] += f"{self.logger_names['mscx']}:{key}"
+        logger_cfg['name'] += f"{self.mscx.logger.logger.name}:{key}"
+        if self.logger.logger.file_handler is not None:
+            logger_cfg['file'] = self.logger.logger.file_handler.baseFilename
         self._detached_annotations[key] = Annotations(df=df, infer_types=self.get_infer_regex(), mscx_obj=self._mscx,
                                                       logger_cfg=logger_cfg)
         if delete:
@@ -540,6 +527,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
             Pass False to not add and not change any label types.
         """
         assert key != 'annotations', "The key 'annotations' is reserved, please choose a different one."
+        assert key is not None, "Key cannot be None."
         assert sum(True for arg in [tsv_path, anno_obj] if arg is not None) == 1, "Pass either tsv_path or anno_obj."
         inf_dict = self.get_infer_regex() if infer else {}
         mscx = None if self._mscx is None else self._mscx
@@ -633,6 +621,21 @@ Use one of the existing keys or load a new set with the method load_annotations(
             #self.logger.error("No file found at this path: " + full_path)
             return None
 
+    @staticmethod
+    def _make_extension_regex(native=True, convertible=True, tsv=False):
+        assert sum((native, convertible)) > 0, "Select at least one type of extensions."
+        exts = []
+        if native:
+            exts.extend(Score.native_formats)
+        if convertible:
+            exts.extend(Score.convertible_formats)
+        if tsv:
+            exts.append('tsv')
+        dot = r'\.'
+        regex = f"({'|'.join(dot + e for e in exts)})$"
+        return re.compile(regex, re.IGNORECASE)
+
+
     def _parse_mscx(self, musescore_file, read_only=False, parser=None, labels_cfg={}):
         """ 
         This method is called by :py:meth:`.__init__` to parse the score. It checks the file extension
@@ -658,23 +661,46 @@ Use one of the existing keys or load a new set with the method load_annotations(
         """
         if parser is not None:
             self.parser = parser
-        if musescore_file[-4:] not in ('mscx', 'mscz'):
-            raise ValueError(f"The extension of a MuseScore file should be mscx or mscz, not {extensions}.")
+
+        permitted_extensions = self.native_formats + self.convertible_formats
+        _, ext = os.path.splitext(musescore_file)
+        ext = ext[1:]
+        if ext.lower() not in permitted_extensions:
+            raise ValueError(f"The extension of a score should be one of {permitted_extensions} not {ext}.")
+        if ext.lower() in self.convertible_formats and self.ms is None:
+            raise ValueError(f"To open a {ext} file, use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.")
         extension = self._handle_path(musescore_file)
         logger_cfg = self.logger_cfg.copy()
         logger_cfg['name'] = self.logger_names[extension]
-        if extension == 'mscz':
-            fake_path = musescore_file[:-4] + 'mscx'
-            self._handle_path(fake_path)
-            with unpack_mscz(musescore_file) as tmp_mscx:
+        musescore_file = resolve_dir(musescore_file)
+
+        if extension in self.convertible_formats +  ('mscz', ):
+            ctxt_mgr = unpack_mscz if extension == 'mscz' else self._tmp_convert
+            with ctxt_mgr(musescore_file) as tmp_mscx:
+                self.logger.debug(f"Using temporary file {os.path.basename(tmp_mscx)} in order to parse {musescore_file}.")
                 self._mscx = MSCX(tmp_mscx, read_only=read_only, labels_cfg=labels_cfg, parser=self.parser,
-                                  logger_cfg=logger_cfg)
+                                  logger_cfg=logger_cfg, parent_score=self)
+                self.mscx.mscx_src = (musescore_file)
         else:
-            self._mscx = MSCX(self.full_paths['mscx'], read_only=read_only, labels_cfg=labels_cfg, parser=self.parser,
-                              logger_cfg=logger_cfg)
+            self._mscx = MSCX(musescore_file, read_only=read_only, labels_cfg=labels_cfg, parser=self.parser,
+                              logger_cfg=logger_cfg, parent_score=self)
         if self.mscx.has_annotations:
             self.mscx._annotations.infer_types(self.get_infer_regex())
 
+    @contextmanager
+    def _tmp_convert(self, file, dir=None):
+        if dir is None:
+            dir = os.path.dirname(file)
+        try:
+            tmp_file = Temp(suffix='.mscx', prefix='.', dir=dir, delete=False)
+            convert(file, tmp_file.name, self.ms, logger=self.logger)
+            yield tmp_file.name
+        except:
+            self.logger.error(f"Error while dealing with the temporarily converted {os.path.basename(file)}")
+            raise
+        finally:
+            os.remove(tmp_file.name)
+
 
     def __repr__(self):
         msg = ''
@@ -716,7 +742,7 @@ Use one of the existing keys or load a new set with the method load_annotations(
 
     def __iter__(self):
         """ Iterate keys of Annotation objects. """
-        attached = ['annotations'] if self._mscx is not None and self.mscx.has_annotations else []
+        attached = ['annotations'] if self._mscx is not None and self._mscx.has_annotations else []
         yield from attached + list(self._detached_annotations.keys())
     # def __setattr__(self, key, value):
     #     assert key != 'annotations', "The key 'annotations' is managed automatically, please pick a different one."
@@ -734,13 +760,16 @@ Use one of the existing keys or load a new set with the method load_annotations(
 ########################################################################################################################
 
 
+
+
+
 class MSCX(LoggedClass):
     """ Object for interacting with the XML structure of a MuseScore 3 file. Is usually attached to a
     :obj:`Score` object and exposed as ``Score.mscx``.
     An object is only created if a score was successfully parsed.
     """
 
-    def __init__(self, mscx_src, read_only=False, parser='bs4', labels_cfg={}, logger_cfg={}, level=None):
+    def __init__(self, mscx_src, read_only=False, parser='bs4', labels_cfg={}, logger_cfg={}, level=None, parent_score=None):
         """ Object for interacting with the XML structure of a MuseScore 3 file.
 
         Parameters
@@ -762,6 +791,8 @@ class MSCX(LoggedClass):
             'file': PATH_TO_LOGFILE to store all log messages under the given path.
         level : :obj:`str` or :obj:`int`
             Quick way to change the logging level which defaults to the one of the parent :obj:`Score`.
+        parent_score : :obj:`Score`, optional
+            Store the Score object to which this MSCX object is attached.
         """
         if level is not None:
             logger_cfg['level'] = level
@@ -770,9 +801,11 @@ class MSCX(LoggedClass):
             self.mscx_src = mscx_src
             """:obj:`str`
             Full path of the parsed MuseScore file."""
+
         else:
             raise ValueError(f"File does not exist: {mscx_src}")
 
+
         self.changed = False
         """:obj:`bool`
         Switches to True as soon as the original XML structure is changed. Does not automatically switch back to False.
@@ -789,6 +822,10 @@ class MSCX(LoggedClass):
         If the score contains at least one <Harmony> tag, this attribute points to the object representing all
         annotations, otherwise it is None."""
 
+        self.parent_score = parent_score
+        """:obj:`Score`
+        The Score object to which this MSCX object is attached."""
+
         self.parser = parser
         """{'bs4'}
         The currently used parser."""
@@ -806,7 +843,19 @@ class MSCX(LoggedClass):
         :py:meth:`Annotations.get_labels()<ms3.annotations.Annotations.get_labels>`.
         """
 
-        self.parse_mscx()
+        ms_version = get_ms_version(self.mscx_src)
+        if ms_version is None:
+            raise ValueError(f"MuseScore version could not be read from {self.mscx_src}")
+        if ms_version[0] == '3':
+            self.parse_mscx()
+        else:
+            if self.parent_score.ms is None:
+                raise ValueError(f"""In order to parse a version {ms_version} file,
+use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.""")
+            with self.parent_score._tmp_convert(self.mscx_src) as tmp:
+                self.logger.debug(f"Using temporally converted file {os.path.basename(tmp)} for parsing the version {ms_version} file.")
+                self.mscx_src = tmp
+                self.parse_mscx()
     #%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END of __init__() %%%%%%%%%%%%%%%%%%%%%%%%%%%%%#
     #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%#
 
@@ -1056,6 +1105,7 @@ class MSCX(LoggedClass):
                 self.logger.warning(f"{target - changes} labels could not be deleted:\n{df.loc[~changed]}")
 
 
+
     def delete_empty_labels(self):
         """ Remove all empty labels from the attached annotations. """
         if self._annotations is None:
