@@ -1,18 +1,85 @@
-import os, re, shutil
+import os, platform, re, shutil, subprocess
 from collections import defaultdict, namedtuple
 from collections.abc import Iterable
 from contextlib import contextmanager
 from fractions import Fraction as frac
 from itertools import repeat, takewhile
+from shutil import which
 from tempfile import NamedTemporaryFile as Temp
 from zipfile import ZipFile as Zip
 
 import pandas as pd
 import numpy as np
 import webcolors
+from pathos import multiprocessing
+from tqdm import tqdm
 
 from .logger import function_logger, update_cfg
 
+DCML_REGEX = re.compile(r"""
+^(\.?
+    ((?P<globalkey>[a-gA-G](b*|\#*))\.)?
+    ((?P<localkey>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
+    ((?P<pedal>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
+    (?P<chord>
+        (?P<numeral>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
+        (?P<form>(%|o|\+|M|\+M))?
+        (?P<figbass>(7|65|43|42|2|64|6))?
+        (\((?P<changes>((\+|-|\^|v)?(b*|\#*)\d)+)\))?
+        (/(?P<relativeroot>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
+    )
+    (?P<pedalend>\])?
+)?
+(?P<phraseend>(\\\\|\}\{|\{|\}))?$
+            """, re.VERBOSE)
+""":obj:`str`
+Constant with a regular expression that recognizes labels conforming to the DCML harmony annotation standard excluding those
+consisting of two alternatives.
+"""
+
+DCML_DOUBLE_REGEX = re.compile(r"""
+                                ^(?P<first>
+                                  (\.?
+                                    ((?P<globalkey>[a-gA-G](b*|\#*))\.)?
+                                    ((?P<localkey>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
+                                    ((?P<pedal>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
+                                    (?P<chord>
+                                        (?P<numeral>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
+                                        (?P<form>(%|o|\+|M|\+M))?
+                                        (?P<figbass>(7|65|43|42|2|64|6))?
+                                        (\((?P<changes>((\+|-|\^|v)?(b*|\#*)\d)+)\))?
+                                        (/(?P<relativeroot>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
+                                    )
+                                    (?P<pedalend>\])?
+                                  )?
+                                  (?P<phraseend>(\\\\|\}\{|\{|\})
+                                  )?
+                                 )
+                                 (-
+                                  (?P<second>
+                                    ((?P<globalkey2>[a-gA-G](b*|\#*))\.)?
+                                    ((?P<localkey2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\.)?
+                                    ((?P<pedal2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i))\[)?
+                                    (?P<chord2>
+                                        (?P<numeral2>(b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i|Ger|It|Fr|@none))
+                                        (?P<form2>(%|o|\+|M|\+M))?
+                                        (?P<figbass2>(7|65|43|42|2|64|6))?
+                                        (\((?P<changes2>((\+|-|\^|v)?(b*|\#*)\d)+)\))?
+                                        (/(?P<relativeroot2>((b*|\#*)(VII|VI|V|IV|III|II|I|vii|vi|v|iv|iii|ii|i)/?)*))?
+                                    )
+                                    (?P<pedalend2>\])?
+                                  )?
+                                  (?P<phraseend2>(\\\\|\}\{|\{|\})
+                                  )?
+                                 )?
+                                $
+                                """,
+                        re.VERBOSE)
+""":obj:`str`
+Constant with a regular expression that recognizes complete labels conforming to the DCML harmony annotation standard 
+including those consisting of two alternatives, without having to split them. It is simply a doubled version of DCML_REGEX.
+"""
+
 
 MS3_HTML = {'#005500': 'ms3_darkgreen',
             '#aa0000': 'ms3_darkred',
@@ -202,7 +269,7 @@ def check_labels(df, regex, column='label', split_regex=None, return_cols=['mc',
     cols = [c for c in return_cols if c in df.columns]
     select_wrong = not_matched.any(axis=1)
     res = check_this.where(not_matched, other='.')[select_wrong]
-    res = res.apply(lambda c: c.str.replace('/', 'empty_harmony'))
+    res = res.apply(lambda c: c.str.replace('^/$', 'empty_harmony'))
     return pd.concat([df.loc[select_wrong, cols], res], axis=1)
 
 
@@ -290,7 +357,19 @@ def commonprefix(paths, sep='/'):
     return sep.join(x[0] for x in takewhile(allnamesequal, bydirectorylevels))
 
 
-def decode_harmonies(df, label_col='label', keep_type=False, return_series=False):
+def compute_mn(df):
+    """ Compute measure numbers from a measure list with columns ['dont_count', 'numbering_offset']
+    """
+    excluded = df['dont_count'].fillna(0).astype(bool)
+    offset = df['numbering_offset']
+    mn = (~excluded).cumsum()
+    if offset.notna().any():
+        offset = offset.fillna(0).astype(int).cumsum()
+        mn += offset
+    return mn.rename('mn')
+
+
+def decode_harmonies(df, label_col='label', keep_type=True, return_series=False):
     df = df.copy()
     drop_cols, compose_label = [], []
     if 'nashville' in df.columns:
@@ -301,19 +380,19 @@ def decode_harmonies(df, label_col='label', keep_type=False, return_series=False
         df.leftParen.replace('/', '(', inplace=True)
         compose_label.append('leftParen')
         drop_cols.append('leftParen')
-    if 'root' in df.columns:
-        df.root = fifths2name(df.root, ms=True)
-        compose_label.append('root')
-        drop_cols.append('root')
+    if 'absolute_root' in df.columns:
+        df.absolute_root = fifths2name(df.absolute_root, ms=True)
+        compose_label.append('absolute_root')
+        drop_cols.append('absolute_root')
         if 'rootCase' in df.columns:
             sel = df.rootCase.notna()
-            df.loc[sel, 'root'] = df.loc[sel, 'root'].str.lower()
+            df.loc[sel, 'absolute_root'] = df.loc[sel, 'absolute_root'].str.lower()
             drop_cols.append('rootCase')
     compose_label.append(label_col)
-    if 'base' in df.columns:
-        df.base = '/' + fifths2name(df.base, ms=True)
-        compose_label.append('base')
-        drop_cols.append('base')
+    if 'absolute_base' in df.columns:
+        df.absolute_base = '/' + fifths2name(df.absolute_base, ms=True)
+        compose_label.append('absolute_base')
+        drop_cols.append('absolute_base')
     if 'rightParen' in df.columns:
         df.rightParen.replace('/', ')', inplace=True)
         compose_label.append('rightParen')
@@ -334,6 +413,74 @@ def decode_harmonies(df, label_col='label', keep_type=False, return_series=False
     return df
 
 
+@function_logger
+def convert(old, new, MS='mscore'):
+    process = [MS, '--appimage-extract-and-run', "-o", new, old] if MS.endswith('.AppImage') else [MS, "-o", new, old]
+    if subprocess.run(process):
+        logger.info(f"Converted {old} to {new}")
+    else:
+        logger.warning("Error while converting " + old)
+
+@function_logger
+def convert_folder(dir, new_folder, extensions=[], target_extension='mscx', regex='.*', suffix=None, recursive=True,
+                   ms='mscore', overwrite=False, parallel=False):
+    """ Convert all files in `dir` that have one of the `extensions` to .mscx format using the executable `MS`.
+
+    Parameters
+    ----------
+    dir, new_folder : str
+        Directories
+    extensions : list, optional
+        If you want to convert only certain formats, give those, e.g. ['mscz', 'xml']
+    recursive : bool, optional
+        Subdirectories as well.
+    MS : str, optional
+        Give the path to the MuseScore executable on your system. Need only if
+        the command 'mscore' does not execute MuseScore on your system.
+    """
+    MS = get_musescore(ms)
+    assert MS is not None, f"MuseScore not found: {ms}"
+    if target_extension[0] == '.':
+        target_extension = target_extension[1:]
+    conversion_params = []
+    #logger.info(f"Traversing {dir} {'' if recursive else 'non-'}recursively...")
+    if len(extensions) > 0:
+        exclude_re = f"^(?:(?!({'|'.join(extensions)})).)*$"
+    else:
+        exclude_re = ''
+    new_dirs = {}
+    for subdir, file in scan_directory(dir, file_re=regex, exclude_re=exclude_re, recursive=recursive, subdirs=True, exclude_files_only=True):
+        if subdir in new_dirs:
+            new_subdir = new_dirs[subdir]
+        else:
+            old_subdir = os.path.relpath(subdir, dir)
+            new_subdir = os.path.join(new_folder, old_subdir) if old_subdir != '.' else new_folder
+            os.makedirs(new_subdir, exist_ok=True)
+            new_dirs[subdir] = new_subdir
+        name, _ = os.path.splitext(file)
+        if suffix is not None:
+            fname = f"{name}{suffix}.{target_extension}"
+        else:
+            fname = f"{name}.{target_extension}"
+        old = os.path.join(subdir, file)
+        new = os.path.join(new_subdir, fname)
+        if overwrite or not os.path.isfile(new):
+            conversion_params.append((old, new, MS))
+        else:
+            logger.debug(new, 'exists already. Pass -o to overwrite.')
+
+
+    # TODO: pass filenames as 'logger' argument to convert()
+    if parallel:
+        pool = multiprocessing.Pool(multiprocessing.cpu_count())
+        pool.starmap(convert, conversion_params)
+        pool.close()
+        pool.join()
+    else:
+        for o, n, ms in conversion_params:
+            convert(o, n, ms)
+
+
 
 def dict2oneliner(d):
     """ Turns a dictionary into a single-line string without brackets."""
@@ -470,6 +617,57 @@ def fifths2str(fifths, steps, inverted=False):
     return acc + steps[fifths % 7]
 
 
+def get_ms_version(mscx_file):
+    with open(mscx_file) as file:
+        for i, l in enumerate(file):
+            if i < 2:
+                pass
+            if i == 2:
+                m = re.search(r"<programVersion>(.*?)</programVersion>", l)
+                if m is None:
+                    return None
+                else:
+                    return m.group(1)
+
+
+@function_logger
+def get_musescore(MS):
+    """ Tests whether a MuseScore executable can be found on the system.
+    Uses: test_binary()
+
+    Parameters
+    ----------
+    MS : :obj:`str`
+        A path to the executable, installed command, or one of the keywords {'auto', 'win', 'mac'}
+
+    Returns
+    -------
+    :obj:`str`
+        Path to the executable if found or None.
+
+    """
+    if MS is None:
+        return MS
+    if MS == 'auto':
+        mapping = {
+            'Windows': 'win',
+            'Darwin': 'mac',
+            'Linux': 'mscore'
+        }
+        system = platform.system()
+        try:
+            MS = mapping[system]
+        except:
+            logger.warning(f"System could not be inferred: {system}")
+            MS = 'mscore'
+    if MS == 'win':
+        program_files = os.environ['PROGRAMFILES']
+        MS = os.path.join(program_files, r"MuseScore 3\bin\MuseScore3.exe")
+    elif MS == 'mac':
+        MS = "/Applications/MuseScore 3.app/Contents/MacOS/mscore"
+    return test_binary(MS, logger=logger)
+
+
 def group_id_tuples(l):
     """ Turns a list of (key, ix) into a {key: [ix]}
 
@@ -597,6 +795,8 @@ def load_tsv(path, index_col=None, sep='\t', converters={}, dtypes={}, stringtyp
         'scalar': safe_frac, }
 
     DTYPES = {
+        'absolute_base': 'Int64',
+        'absolute_root': 'Int64',
         'alt_label': str,
         'barline': str,
         'base': 'Int64',
@@ -960,41 +1160,70 @@ def rgba2params(named_tuple):
     return {'color_'+k: v for k, v in attrs.items()}
 
 
-def scan_directory(dir, file_re=r".*", folder_re=r".*", exclude_re=r"^(\.|_)", recursive=True):
-    """ Get a list of files.
+
+@function_logger
+def scan_directory(directory, file_re=r".*", folder_re=r".*", exclude_re=r"^(\.|_)", recursive=True, subdirs=False, progress=False, exclude_files_only=False):
+    """ Generator of file names in ``directory``.
 
     Parameters
     ----------
     dir : :obj:`str`
         Directory to be scanned for files.
-    file_re, folder_re : :obj:`str`, optional
+    file_re, folder_re : :obj:`str` or :obj:`re.Pattern`, optional
         Regular expressions for filtering certain file names or folder names.
         The regEx are checked with search(), not match(), allowing for fuzzy search.
     recursive : :obj:`bool`, optional
         By default, sub-directories are recursively scanned. Pass False to scan only ``dir``.
+    subdirs : :obj:`bool`, optional
+        By default, full file paths are returned. Pass True to return (path, name) tuples instead.
+    progress : :obj:`bool`, optional
+        By default, the scanning process is shown. Pass False to prevent.
+    exclude_files_only : :obj:`bool`, optional
+        By default, ``exclude_re`` excludes files and folder. Pass True to exclude only files matching the regEx.
 
-    Returns
-    -------
+
+    Yields
+    ------
     list
         List of full paths meeting the criteria.
 
     """
-    def check_regex(reg, s):
-        res = re.search(reg, s) is not None and re.search(exclude_re, s) is None
-        return res
+    def traverse(dir):
+        nonlocal counter
 
-    res = []
-    for subdir, dirs, files in os.walk(dir):
-        _, current_folder = os.path.split(subdir)
-        if recursive and check_regex('', current_folder):
-            dirs[:] = [d for d in sorted(dirs)]
-        else:
-            dirs[:] = []
-        if check_regex(folder_re, current_folder):
-            files = [os.path.join(subdir, f) for f in sorted(files) if check_regex(file_re, f)]
-            res.extend(files)
-    return res
+        def check_regex(reg, s, excl=exclude_re):
+            try:
+                res = re.search(reg, s) is not None and re.search(excl, s) is None
+            except:
+                print(reg)
+                raise
+            return res
 
+        for dir_entry in os.scandir(dir):
+            name = dir_entry.name
+            path = os.path.join(dir, name)
+            if dir_entry.is_dir() and recursive:
+                if (exclude_files_only and check_regex(folder_re, name, excl='^$')) or (not exclude_files_only and check_regex(folder_re, name)):
+                    for res in traverse(path):
+                        yield res
+            else:
+                if pbar is not None:
+                    pbar.update()
+                if dir_entry.is_file() and check_regex(file_re, name):
+                    counter += 1
+                    if pbar is not None:
+                        pbar.set_postfix({'selected': counter})
+                    if subdirs:
+                        yield (dir, name)
+                    else:
+                        yield path
+
+    directory = resolve_dir(directory)
+    counter = 0
+    if not os.path.isdir(directory):
+        logger.warning("Not an existing directory: " + directory)
+    pbar = tqdm(desc='Scanning files', unit=' files') if progress else None
+    return traverse(directory)
 
 
 def sort_tpcs(tpcs, ascending=True, start=None):
@@ -1022,7 +1251,7 @@ def sort_tpcs(tpcs, ascending=True, start=None):
 
 
 @function_logger
-def split_alternatives(df, column='label', regex=r"-(?!(\d|b+\d|\#))", max=2, inplace=False, alternatives_only=False):
+def split_alternatives(df, column='label', regex=r"-(?!(\d|b+\d|\#+\d))", max=2, inplace=False, alternatives_only=False):
     """
     Splits labels that come with an alternative separated by '-' and adds
     a new column. Only one alternative is taken into account. `df` is
@@ -1110,6 +1339,21 @@ def string2lines(string, length=80):
     return '\n'.join(chunkstring(string, length))
 
 
+@function_logger
+def test_binary(command):
+    if command is None:
+        return command
+    if os.path.isfile(command):
+        logger.debug(f"Found MuseScore binary: {command}")
+        return command
+    if which(command) is None:
+        logger.warning(f"MuseScore binary not found and not an installed command: {command}")
+        return None
+    else:
+        logger.debug(f"Found MuseScore command: {command}")
+        return command
+
+
 def transform(df, func, param2col=None, column_wise=False, **kwargs):
     """ Compute a function for every row of a DataFrame, using several cols as arguments.
         The result is the same as using df.apply(lambda r: func(param1=r.col1, param2=r.col2...), axis=1)
@@ -1177,19 +1421,45 @@ def transform(df, func, param2col=None, column_wise=False, **kwargs):
     return res
 
 
+def unfold_repeats(df, mc_sequence):
+    """ Use a succesion of MCs to bring a DataFrame in this succession. MCs may repeat.
+
+    Parameters
+    ----------
+    df : :obj:`pandas.DataFrame`
+        DataFrame needs to have the columns 'mc' and 'mn'.
+    mc_sequence : :obj:`pandas.Series`
+        A Series of the format ``{playthrough: mc}`` where ``playthrough`` corresponds
+        to continuous MN
+    """
+    vc = df.mc.value_counts()
+    res = df.set_index('mc')
+    seq = mc_sequence[mc_sequence.isin(res.index)]
+    playthrough_col = sum([[playthrough] * vc[mc] for playthrough, mc in seq.items()], [])
+    res = res.loc[seq.values]
+    res.insert(res.columns.get_loc('mn') + 1, 'playthrough', playthrough_col)
+    return res.reset_index()
+
+
 @contextmanager
 def unpack_mscz(mscz, dir=None):
     if dir is None:
         dir = os.path.dirname(mscz)
     tmp_file = Temp(suffix='.mscx', prefix='.', dir=dir, delete=False)
     with Zip(mscz) as zip_file:
-        mscx = next(fname for fname in zip_file.namelist() if fname.endswith('.mscx'))
+        mscx_files = [f for f in zip_file.namelist() if f.endswith('.mscx')]
+        if len(mscx_files) > 1:
+            logger.info(f"{mscz} contains several MSCX files. Picking the first one")
+        mscx = mscx_files[0]
         with zip_file.open(mscx) as mscx_file:
             with tmp_file as tmp:
                 for line in mscx_file:
                     tmp.write(line)
     try:
         yield tmp_file.name
+    except:
+        logger.error(f"Error while dealing with the temporarily unpacked {os.path.basename(mscz)}")
+        raise
     finally:
         os.remove(tmp_file.name)
 
